@@ -51,14 +51,14 @@ enum
 
 static void exo_wrap_table_class_init         (ExoWrapTableClass  *klass);
 static void exo_wrap_table_init               (ExoWrapTable       *table);
-static void exo_wrap_table_get_property       (GObject            *object,
-                                               guint               prop_id,
-                                               GValue             *value,
-                                               GParamSpec         *pspec);
-static void exo_wrap_table_set_property       (GObject            *object,
-                                               guint               prop_id,
-                                               const GValue       *value,
-                                               GParamSpec         *pspec);
+//static void exo_wrap_table_get_property       (GObject            *object,
+//                                               guint               prop_id,
+//                                               GValue             *value,
+//                                               GParamSpec         *pspec);
+//static void exo_wrap_table_set_property       (GObject            *object,
+//                                               guint               prop_id,
+//                                               const GValue       *value,
+//                                               GParamSpec         *pspec);
 static void exo_wrap_table_size_request       (GtkWidget          *widget,
                                                GtkRequisition     *requisition);
 static void exo_wrap_table_size_allocate      (GtkWidget          *widget,
@@ -100,6 +100,32 @@ struct _ExoWrapTablePrivate
 static GObjectClass *exo_wrap_table_parent_class;
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+static void
+exo_wrap_table_get_preferred_width (GtkWidget *widget,
+                               gint      *minimal_width,
+                               gint      *natural_width)
+{
+  GtkRequisition requisition;
+
+  exo_wrap_table_size_request (widget, &requisition);
+
+  *minimal_width = *natural_width = requisition.width;
+}
+
+static void
+exo_wrap_table_get_preferred_height (GtkWidget *widget,
+                                gint      *minimal_height,
+                                gint      *natural_height)
+{
+  GtkRequisition requisition;
+
+  exo_wrap_table_size_request (widget, &requisition);
+
+  *minimal_height = *natural_height = requisition.height;
+}
+#endif
+
 
 GType
 exo_wrap_table_get_type (void)
@@ -112,9 +138,9 @@ exo_wrap_table_get_type (void)
       type = g_type_register_static_simple (GTK_TYPE_CONTAINER,
                                           "ExoWrapTable",
                                           sizeof (ExoWrapTableClass),
-                                          exo_wrap_table_class_init,
+                                          (GClassInitFunc)exo_wrap_table_class_init,
                                           sizeof (ExoWrapTable),
-                                          exo_wrap_table_init, 0);
+                                          (GInstanceInitFunc)exo_wrap_table_init, 0);
     }
 
   return type;
@@ -127,7 +153,7 @@ exo_wrap_table_class_init (ExoWrapTableClass *klass)
 {
   GtkContainerClass *gtkcontainer_class;
   GtkWidgetClass    *gtkwidget_class;
-  GObjectClass      *gobject_class;
+//  GObjectClass      *gobject_class;
 
   /* add our private data to the class */
   g_type_class_add_private (klass, sizeof (ExoWrapTablePrivate));
@@ -135,15 +161,19 @@ exo_wrap_table_class_init (ExoWrapTableClass *klass)
   /* determine our parent type class */
   exo_wrap_table_parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class = G_OBJECT_CLASS (klass);
 /*
+  gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->get_property = exo_wrap_table_get_property;
   gobject_class->set_property = exo_wrap_table_set_property;
 */
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
+#if GTK_CHECK_VERSION(3, 0, 0)
+  gtkwidget_class->get_preferred_width = exo_wrap_table_get_preferred_width;
+  gtkwidget_class->get_preferred_height = exo_wrap_table_get_preferred_height;
+#else
   gtkwidget_class->size_request = exo_wrap_table_size_request;
+#endif
   gtkwidget_class->size_allocate = exo_wrap_table_size_allocate;
-
   gtkcontainer_class = GTK_CONTAINER_CLASS (klass);
   gtkcontainer_class->add = exo_wrap_table_add;
   gtkcontainer_class->remove = exo_wrap_table_remove;
@@ -209,7 +239,11 @@ exo_wrap_table_init (ExoWrapTable *table)
   table->priv = EXO_WRAP_TABLE_GET_PRIVATE (table);
 
   /* we don't provide our own window */
+#if GTK_CHECK_VERSION(2, 18, 0)
+  gtk_widget_set_has_window(GTK_WIDGET(table), FALSE);
+#else
   GTK_WIDGET_SET_FLAGS (table, GTK_NO_WINDOW);
+#endif
 }
 
 
@@ -291,9 +325,17 @@ exo_wrap_table_size_request (GtkWidget      *widget,
   /* check if we have any visible children */
   if (G_LIKELY (num_children > 0))
     {
+#if GTK_CHECK_VERSION(2,18,0)
+      GtkAllocation allocation;
+      gtk_widget_get_allocation(GTK_WIDGET(widget), &allocation);
+      num_cols = exo_wrap_table_get_num_fitting (allocation.width
+                                                 - gtk_container_get_border_width(GTK_CONTAINER (widget)) * 2,
+                                                 table->priv->col_spacing, max_width);
+#else
       num_cols = exo_wrap_table_get_num_fitting (widget->allocation.width
                                                  - GTK_CONTAINER (widget)->border_width * 2,
                                                  table->priv->col_spacing, max_width);
+#endif
       num_rows = num_children / num_cols;
       num_rows = MAX (num_rows, 1);
 
@@ -303,7 +345,11 @@ exo_wrap_table_size_request (GtkWidget      *widget,
       requisition->width = -1;
       requisition->height = (num_rows * max_height)
                          + (num_rows - 1) * table->priv->col_spacing
+#if GTK_CHECK_VERSION(2,18,0)
+                         + gtk_container_get_border_width(GTK_CONTAINER (widget)) * 2;
+#else
                          + GTK_CONTAINER (widget)->border_width * 2;
+#endif
     }
   else
     {
@@ -321,7 +367,11 @@ exo_wrap_table_size_allocate (GtkWidget     *widget,
   ExoWrapTable *table = EXO_WRAP_TABLE (widget);
 
   /* setup the new allocation */
+#if GTK_CHECK_VERSION(2,18,0)
+  gtk_widget_set_allocation(widget, allocation);
+#else
   widget->allocation = *allocation;
+#endif
 
   /* layout the children */
   exo_wrap_table_layout (table);
@@ -334,14 +384,26 @@ exo_wrap_table_add (GtkContainer *container,
                     GtkWidget    *widget)
 {
   ExoWrapTable *table = EXO_WRAP_TABLE (container);
+  GtkWidget *w = GTK_WIDGET (container);
 
   /* take over ownership */
-  gtk_widget_set_parent (widget, GTK_WIDGET (table));
+  gtk_widget_set_parent (widget, w);
 
   /* add the child to our internal list */
   table->priv->children = g_list_append (table->priv->children, widget);
 
   /* realize the widget if required */
+#if GTK_CHECK_VERSION(2,20,0)
+  if (gtk_widget_get_realized(w))
+    gtk_widget_realize (widget);
+
+  /* map the widget if required */
+  if (gtk_widget_get_visible(w) && gtk_widget_get_visible(widget))
+    {
+      if (gtk_widget_get_mapped(w))
+        gtk_widget_map (widget);
+    }
+#else
   if (GTK_WIDGET_REALIZED (container))
     gtk_widget_realize (widget);
 
@@ -351,9 +413,10 @@ exo_wrap_table_add (GtkContainer *container,
       if (GTK_WIDGET_MAPPED (container))
         gtk_widget_map (widget);
     }
+#endif
 
   /* queue a resize on the table */
-  gtk_widget_queue_resize (GTK_WIDGET (container));
+  gtk_widget_queue_resize (w);
 }
 
 
@@ -366,7 +429,11 @@ exo_wrap_table_remove (GtkContainer *container,
   gboolean      widget_was_visible;
 
   /* check if the widget was visible */
+#if GTK_CHECK_VERSION(2,18,0)
+  widget_was_visible = gtk_widget_get_visible(widget);
+#else
   widget_was_visible = GTK_WIDGET_VISIBLE (widget);
+#endif
 
   /* unparent and remove the widget */
   gtk_widget_unparent (widget);
@@ -416,6 +483,10 @@ exo_wrap_table_layout (ExoWrapTable *table)
   gint           num_cols;
   gint           max_height;
   gint           max_width;
+#if GTK_CHECK_VERSION(2,18,0)
+  GtkAllocation  allocation;
+  gtk_widget_get_allocation(GTK_WIDGET(table), &allocation);
+#endif
 
   /* determine the number of visible children and the max size */
   num_children = exo_wrap_table_get_max_child_size (table, &max_width, &max_height);
@@ -423,9 +494,15 @@ exo_wrap_table_layout (ExoWrapTable *table)
     return;
 
   /* determine the number of columns */
+#if GTK_CHECK_VERSION(2,18,0)
+  num_cols = exo_wrap_table_get_num_fitting (allocation.width
+                                             - gtk_container_get_border_width(GTK_CONTAINER (table)) * 2,
+                                             table->priv->col_spacing, max_width);
+#else
   num_cols = exo_wrap_table_get_num_fitting (GTK_WIDGET (table)->allocation.width
                                              - GTK_CONTAINER (table)->border_width * 2,
                                              table->priv->col_spacing, max_width);
+#endif
 
   /* verify that the number of columns match */
   if (G_UNLIKELY (num_cols != table->priv->num_cols))
@@ -436,19 +513,32 @@ exo_wrap_table_layout (ExoWrapTable *table)
     }
 
   /* determine the horizontal bounds */
+#if GTK_CHECK_VERSION(2,18,0)
+  x0 = allocation.x + gtk_container_get_border_width(GTK_CONTAINER (table));
+  x1 = x0 + allocation.width - gtk_container_get_border_width(GTK_CONTAINER (table));
+#else
   x0 = GTK_WIDGET (table)->allocation.x + GTK_CONTAINER (table)->border_width;
   x1 = x0 + GTK_WIDGET (table)->allocation.width - GTK_CONTAINER (table)->border_width;
+#endif
 
   /* initialize the position */
   x = x0;
+#if GTK_CHECK_VERSION(2,18,0)
+  y = allocation.y + gtk_container_get_border_width(GTK_CONTAINER (table));
+#else
   y = GTK_WIDGET (table)->allocation.y + GTK_CONTAINER (table)->border_width;
+#endif
 
   /* allocate space to all visible children */
   for (lp = table->priv->children; lp != NULL; lp = lp->next)
     {
       /* allocate space only for visible children */
       child = GTK_WIDGET (lp->data);
+#if GTK_CHECK_VERSION(2,18,0)
+      if (G_UNLIKELY (!gtk_widget_get_visible(child)))
+#else
       if (G_UNLIKELY (!GTK_WIDGET_VISIBLE (child)))
+#endif
         continue;
 
       /* initialize the child position */
@@ -519,7 +609,11 @@ exo_wrap_table_get_max_child_size (const ExoWrapTable *table,
   for (lp = table->priv->children; lp != NULL; lp = lp->next)
     {
       child = GTK_WIDGET (lp->data);
+#if GTK_CHECK_VERSION(2,18,0)
+      if (gtk_widget_get_visible(child))
+#else
       if (GTK_WIDGET_VISIBLE (child))
+#endif
         {
           gtk_widget_size_request (child, &child_requisition);
           if (child_requisition.width > max_width)
